From 737e5d0abff68e411cae6ca3ba9a856d1b5e7ff5 Mon Sep 17 00:00:00 2001
From: Masahiko Sawada <sawada.mshk@gmail.com>
Date: Tue, 19 May 2020 15:06:38 +0900
Subject: [PATCH v37 8/9] Documentation update.

Co-authored-by: Masahiko Sawada, Ashutosh Bapat
---
 doc/src/sgml/catalogs.sgml                | 135 ++++++++++++
 doc/src/sgml/config.sgml                  | 144 +++++++++++++
 doc/src/sgml/distributed-transaction.sgml | 158 ++++++++++++++
 doc/src/sgml/fdwhandler.sgml              | 245 ++++++++++++++++++++++
 doc/src/sgml/filelist.sgml                |   1 +
 doc/src/sgml/func.sgml                    | 147 +++++++++++++
 doc/src/sgml/monitoring.sgml              |  42 ++++
 doc/src/sgml/postgres.sgml                |   1 +
 doc/src/sgml/storage.sgml                 |   6 +
 src/backend/access/transam/README.fdwxact | 142 +++++++++++++
 10 files changed, 1021 insertions(+)
 create mode 100644 doc/src/sgml/distributed-transaction.sgml
 create mode 100644 src/backend/access/transam/README.fdwxact

diff --git a/doc/src/sgml/catalogs.sgml b/doc/src/sgml/catalogs.sgml
index f517a7d4af..b8d6b54e7b 100644
--- a/doc/src/sgml/catalogs.sgml
+++ b/doc/src/sgml/catalogs.sgml
@@ -9388,6 +9388,11 @@ SCRAM-SHA-256$<replaceable>&lt;iteration count&gt;</replaceable>:<replaceable>&l
       <entry>summary of configuration file contents</entry>
      </row>
 
+     <row>
+      <entry><link linkend="view-pg-foreign-xacts"><structname>pg_foreign_xacts</structname></link></entry>
+      <entry>foreign transactions</entry>
+     </row>
+
      <row>
       <entry><link linkend="view-pg-group"><structname>pg_group</structname></link></entry>
       <entry>groups of database users</entry>
@@ -11262,6 +11267,136 @@ SELECT * FROM pg_locks pl LEFT JOIN pg_prepared_xacts ppx
 
  </sect1>
 
+ <sect1 id="view-pg-foreign-xacts">
+  <title><structname>pg_foreign_xacts</structname></title>
+
+  <indexterm zone="view-pg-foreign-xacts">
+   <primary>pg_foreign_xacts</primary>
+  </indexterm>
+
+  <para>
+   The view <structname>pg_foreign_xacts</structname> displays
+   information about foreign transactions that are opened on
+   foreign servers for atomic distributed transaction commit (see
+   <xref linkend="atomic-commit"/> for details).
+  </para>
+
+  <para>
+   <structname>pg_foreign_xacts</structname> contains one row per foreign
+   transaction.  An entry is removed when the foreign transaction is
+   committed or rolled back.
+  </para>
+
+  <table>
+   <title><structname>pg_foreign_xacts</structname> Columns</title>
+
+   <tgroup cols="4">
+    <thead>
+     <row>
+      <entry>Name</entry>
+      <entry>Type</entry>
+      <entry>References</entry>
+      <entry>Description</entry>
+     </row>
+    </thead>
+    <tbody>
+     <row>
+      <entry><structfield>dbid</structfield></entry>
+      <entry><type>oid</type></entry>
+      <entry><literal><link linkend="catalog-pg-database"><structname>pg_database</structname></link>.oid</literal></entry>
+      <entry>
+       OID of the database which the foreign transaction resides in
+      </entry>
+     </row>
+     <row>
+      <entry><structfield>xid</structfield></entry>
+      <entry><type>xid</type></entry>
+      <entry></entry>
+      <entry>
+       Numeric transaction identifier with which this foreign transaction
+       associates
+      </entry>
+     </row>
+     <row>
+      <entry><structfield>serverid</structfield></entry>
+      <entry><type>oid</type></entry>
+      <entry><literal><link linkend="catalog-pg-foreign-server"><structname>pg_foreign_server</structname></link>.oid</literal></entry>
+      <entry>
+       The OID of the foreign server on which the foreign transaction is prepared
+      </entry>
+     </row>
+     <row>
+      <entry><structfield>userid</structfield></entry>
+      <entry><type>oid</type></entry>
+      <entry><literal><link linkend="view-pg-user"><structname>pg_user</structname></link>.oid</literal></entry>
+      <entry>
+       The OID of the user that prepared this foreign transaction.
+      </entry>
+     </row>
+     <row>
+      <entry><structfield>status</structfield></entry>
+      <entry><type>text</type></entry>
+      <entry></entry>
+      <entry>
+       Status of foreign transaction. Possible values are:
+       <itemizedlist>
+        <listitem>
+         <para>
+          <literal>preparing</literal> : This foreign transaction is being prepared.
+         </para>
+        </listitem>
+        <listitem>
+         <para>
+          <literal>prepared</literal> : This foreign transaction has been prepared.
+         </para>
+        </listitem>
+        <listitem>
+         <para>
+          <literal>comitting</literal> : This foreign transcation has been
+          prepared to commit or being committed.
+         </para>
+        </listitem>
+        <listitem>
+         <para>
+          <literal>abortin</literal> : This foreign transaction has been
+          prepared to abort or being aborted.
+         </para>
+        </listitem>
+       </itemizedlist>
+      </entry>
+     </row>
+     <row>
+      <entry><structfield>locker_pid</structfield></entry>
+      <entry><type>int</type></entry>
+      <entry></entry>
+      <entry>
+       Process ID of the locker currently processing.
+      </entry>
+     </row>
+     <row>
+      <entry><structfield>identifier</structfield></entry>
+      <entry><type>text</type></entry>
+      <entry></entry>
+      <entry>
+       The identifier of the prepared foreign transaction.
+      </entry>
+     </row>
+    </tbody>
+   </tgroup>
+  </table>
+
+  <para>
+   When the <structname>pg_foreign_xacts</structname> view is accessed, the
+   internal transaction manager data structures are momentarily locked, and
+   a copy is made for the view to display.  This ensures that the
+   view produces a consistent set of results, while not blocking
+   normal operations longer than necessary.  Nonetheless
+   there could be some impact on database performance if this view is
+   frequently accessed.
+  </para>
+
+ </sect1>
+
  <sect1 id="view-pg-publication-tables">
   <title><structname>pg_publication_tables</structname></title>
 
diff --git a/doc/src/sgml/config.sgml b/doc/src/sgml/config.sgml
index 6098f6b020..951fbef695 100644
--- a/doc/src/sgml/config.sgml
+++ b/doc/src/sgml/config.sgml
@@ -9580,6 +9580,150 @@ dynamic_library_path = 'C:\tools\postgresql;H:\my_project\lib;$libdir'
      </variablelist>
    </sect1>
 
+   <sect1 id="runtime-config-distributed-transaction">
+    <title>Distributed Transaction Management</title>
+
+    <sect2 id="runtime-config-distributed-transaction-settings">
+     <title>Setting</title>
+     <variablelist>
+
+      <varlistentry id="guc-foreign-twophase-commit" xreflabel="foreign_twophse_commit">
+       <term><varname>foreign_twophase_commit</varname> (<type>enum</type>)
+        <indexterm>
+         <primary><varname>foreign_twophase_commit</varname> configuration parameter</primary>
+        </indexterm>
+       </term>
+       <listitem>
+        <para>
+         Specifies whether distributed transaction commits ensures that all
+         involved changes on foreign servers are committed or not. Valid
+         values are <literal>required</literal> and <literal>disabled</literal>.
+         The default setting is <literal>disabled</literal>. Setting to
+         <literal>disabled</literal> don't use two-phase commit protocol to
+         commit or rollback distributed transactions. When set to
+         <literal>required</literal> distributed transactions strictly requires
+         that all written servers can use two-phase commit protocol.  That is,
+         the distributed transaction cannot commit if even one server does not
+         support the prepare callback routine
+         (described in <xref linkend="fdw-callbacks-transaction-management"/>).
+         In <literal>required</literal> case, distributed transaction commit will
+         wait for all involving foreign transaction to be committed before the
+         command return a "success" indication to the client.
+        </para>
+
+        <para>
+         This parameter can be changed at any time; the behavior for any one
+         transaction is determined by the setting in effect when it commits.
+        </para>
+
+        <note>
+         <para>
+          When <literal>disabled</literal> there can be risk of database
+          consistency if one or more foreign servers crashes while committing
+          the distributed transactions.
+         </para>
+        </note>
+       </listitem>
+      </varlistentry>
+
+      <varlistentry id="guc-max-prepared-foreign-transactions" xreflabel="max_prepared_foreign_transactions">
+       <term><varname>max_prepared_foreign_transactions</varname> (<type>integer</type>)
+        <indexterm>
+         <primary><varname>max_prepared_foreign_transactions</varname> configuration parameter</primary>
+        </indexterm>
+       </term>
+       <listitem>
+        <para>
+         Sets the maximum number of foreign transactions that can be prepared
+         simultaneously. A single local transaction can give rise to multiple
+         foreign transaction. If a user expects <literal>N</literal> local
+         transactions and each of those involves <literal>K</literal> foreign
+         servers, this value need to be set <literal>N * K</literal>, not
+         just <literal>N</literal>.  This parameter can only be set at server
+         start.
+        </para>
+        <para>
+         When running a standby server, you must set this parameter to the
+         same or higher value than on the master server. Otherwise, queries
+         will not be allowed in the standby server.
+        </para>
+       </listitem>
+      </varlistentry>
+
+     </variablelist>
+    </sect2>
+
+    <sect2 id="runtime-config-foreign-transaction-resolver">
+     <title>Foreign Transaction Resolvers</title>
+
+     <para>
+      These settings control the behavior of a foreign transaction resolver.
+     </para>
+
+     <variablelist>
+      <varlistentry id="guc-max-foreign-transaction-resolvers" xreflabel="max_foreign_transaction_resolvers">
+       <term><varname>max_foreign_transaction_resolvers</varname> (<type>int</type>)
+        <indexterm>
+         <primary><varname>max_foreign_transaction_resolvers</varname> configuration parameter</primary>
+        </indexterm>
+       </term>
+       <listitem>
+        <para>
+         Specifies maximum number of foreign transaction resolution workers. A foreign transaction
+         resolver is responsible for foreign transaction resolution on one database.
+        </para>
+        <para>
+         Foreign transaction resolution workers are taken from the pool defined by
+         <varname>max_worker_processes</varname>.
+        </para>
+        <para>
+         The default value is 0.
+        </para>
+       </listitem>
+      </varlistentry>
+
+      <varlistentry id="guc-foreign-transaction-resolution-rety-interval" xreflabel="foreign_transaction_resolution_retry_interval">
+       <term><varname>foreign_transaction_resolution_retry_interval</varname> (<type>integer</type>)
+        <indexterm>
+         <primary><varname>foreign_transaction_resolution_interval</varname> configuration parameter</primary>
+        </indexterm>
+       </term>
+       <listitem>
+        <para>
+         Specify how long the foreign transaction resolver should wait when the last resolution
+         fails before retrying to resolve foreign transaction. This parameter can only be set in the
+         <filename>postgresql.conf</filename> file or on the server command line.
+        </para>
+        <para>
+         The default value is 10 seconds.
+        </para>
+       </listitem>
+      </varlistentry>
+
+      <varlistentry id="guc-foreign-transaction-resolver-timeout" xreflabel="foreign_transaction_resolver_timeout">
+       <term><varname>foreign_transaction_resolver_timeout</varname> (<type>integer</type>)
+        <indexterm>
+         <primary><varname>foreign_transaction_resolver_timeout</varname> configuration parameter</primary>
+        </indexterm>
+       </term>
+       <listitem>
+        <para>
+         Terminate foreign transaction resolver processes that don't have any foreign
+         transactions to resolve longer than the specified number of milliseconds.
+         A value of zero disables the timeout mechanism, meaning it connects to one
+         database until stopping manually by <function>pg_stop_foreign_xact_resovler()</function>.
+         This parameter can only be set in the <filename>postgresql.conf</filename>
+         file or on the server command line.
+        </para>
+        <para>
+         The default value is 60 seconds.
+        </para>
+       </listitem>
+      </varlistentry>
+     </variablelist>
+    </sect2>
+   </sect1>
+
    <sect1 id="runtime-config-compatible">
     <title>Version and Platform Compatibility</title>
 
diff --git a/doc/src/sgml/distributed-transaction.sgml b/doc/src/sgml/distributed-transaction.sgml
new file mode 100644
index 0000000000..1106fe00c9
--- /dev/null
+++ b/doc/src/sgml/distributed-transaction.sgml
@@ -0,0 +1,158 @@
+<!-- doc/src/sgml/distributed-transaction.sgml -->
+
+<chapter id="distributed-transaction">
+ <title>Distributed Transaction</title>
+
+ <para>
+  A distributed transaction is a transaction in which two or more network hosts
+  are involved. <productname>PostgreSQL</productname>'s global transaction
+  manager supports distributed transactions that access foreign servers using
+  Foreign Data Wrappers.
+ </para>
+
+ <sect1 id="atomic-commit">
+  <title>Atomic Commit</title>
+
+  <para>
+   Formerly, transactions on foreign servers were simply committed or rolled
+   back one by one. Therefore, when one foreign server had a problem during
+   commit, it was possible that transactions on only part of foreign servers
+   are committed while other transactions are rolled back. This used to leave
+   database data in an inconsistent state in terms of a federated database.
+   Atomic commit of distributed transactions is an operation that applies a set
+   of changes as a single operation globally. This guarantees all-or-nothing
+   results for the changes on all remote hosts involved in.
+   <productname>PostgreSQL</productname> provides a way to perform read-write
+   transactions with foreign resources using foreign data wrappers.
+   Using <productname>PostgreSQL</productname>'s atomic commit ensures that
+   all changes on foreign servers are either committed or rolled back using the
+   transaction callback routines
+   (see <xref linkend="fdw-callbacks-transaction-management"/>).
+  </para>
+
+  <sect2>
+   <title>Atomic Commit Using Two-phase Commit Protocol</title>
+
+   <para>
+    To achieve commit among all foreign servers automatically,
+    <productname>PostgreSQL</productname> employs two-phase commit protocol,
+    which is a type of atomic commitment protocol (ACP).  Using two-phase
+    commit protocol, the commit sequence of distributed transaction performs
+    with the following steps:
+    <orderedlist>
+     <listitem>
+      <para>
+       Prepare all transactions on foreign servers.
+       <productname>PostgreSQL</productname>'s distributed transaction manager
+       prepares all transactions on the foreign servers if two-phase commit is
+       required. Two-phase commit is required when the transaction modifies
+       data on two or more servers including the local server itself and
+       <xref linkend="guc-foreign-twophase-commit"/> is
+       <literal>required</literal>. If the preparation on all foreign servers
+       is successful then go to the next step.  If there is any failure in the
+       prepare phase, the server will rollback all the transactions on both
+       local and foreign servers.
+      </para>
+     </listitem>
+     <listitem>
+      <para>
+       Commit the local transaction. The server commits the transaction locally.
+       Once the local transaction gets committed, we will never rollback any
+       involved transactions.
+      </para>
+     </listitem>
+     <listitem>
+      <para>
+       Resolve all prepared transactions on foreign servers. Prepared transactions
+       are committed or rolled back according to the result of the local transaction.
+       This step is performed by a foreign transaction resolver process.
+      </para>
+     </listitem>
+    </orderedlist>
+   </para>
+
+   <para>
+    The above sequence is executed transparently to the users at transaction commit.
+    The transaction returns an acknowledgment of the successful commit of the
+    distributed transaction to the client after step 2.  After that, all
+    prepared transactions are resolved asynchronously by a foreign transaction
+    resolver process.
+   </para>
+
+   <para>
+    When the user executes <command>PREPARE TRANSACTION</command>, the transaction
+    prepares the local transactions as well as all involved transactions on the
+    foreign servers. Likewise, when <command>COMMIT PREPARED</command> or
+    <command>ROLLBACK PREPARED</command> all prepared transactions are resolved
+    asynchronously after committing or rolling back the local transaction.
+   </para>
+  </sect2>
+
+  <sect2 id="atomic-commit-in-doubt-transaction">
+   <title>In-Doubt Transactions</title>
+
+   <para>
+    Distributed transaction can become <firstterm>in-doubt</firstterm> state
+    after preparing the all involved transactions until the all involved
+    transaction are resolved.  In case where the local node crashes during
+    preparing transactions, the distributed transaction becomes in-doubt
+    state.  The information of involved foreign transactions is recovered
+    during crash recovery and these are resolved in background.  Until all
+    in-doubt state transactions are resolved, other transactions might see
+    an inconsistent results on the foreign servers on reading.
+   </para>
+
+   <para>
+    The foreign transaction resolver processes automatically resolve the
+    transactions associated with the in-doubt distributed transaction. Or you
+    can use <function>pg_resolve_foriegn_xact</function> function to resolve
+    it manually.
+   </para>
+  </sect2>
+
+  <sect2 id="atomic-commit-transaction-resolver">
+   <title>Foreign Transaction Resolver Processes</title>
+
+   <para>
+    Foreign transaction resolver processes are auxiliary processes that are
+    responsible for resolving in-doubt distributed transactions. They commit or
+    rollback prepared transactions on all foreign servers involved with the
+    distributed transaction according to the result of the corresponding local
+    transaction.
+   </para>
+
+   <para>
+    One foreign transaction resolver is responsible for transaction resolutions
+    on the database to which it is connected. On failure during resolution, they
+    retry to resolve at an interval of
+    <varname>foreign_transaction_resolution_interval</varname> time.
+   </para>
+
+   <note>
+    <para>
+     During a foreign transaction resolver process connecting to the database,
+     the database cannot be dropped without immediate shutdown. You can call
+     <function>pg_stop_foreign_xact_resovler</function> function to stop the
+     particular resolver process before dropping the database.
+    </para>
+   </note>
+  </sect2>
+
+  <sect2>
+   <title>Configuration Settings</title>
+
+   <para>
+    Atomic commit requires several configuration options to be set.
+    On the local node, <xref linkend="guc-max-prepared-foreign-transactions"/> and
+    <xref linkend="guc-max-foreign-transaction-resolvers"/> must be non-zero value,
+    and <xref linkend="guc-foreign-twophase-commit"/> must be enabled.  Additionally,
+    the <varname>max_worker_processes</varname> may need to be adjusted
+    to accommodate for foreign transaction resolver workers, at least
+    (<varname>max_foreign_transaction_resolvers</varname> + <literal>1</literal>).
+    Note that other <productname>PostgreSQL</productname> features such as parallel
+    queries, logical replication, etc., also take worker slots from
+    <varname>max_worker_processes</varname>.
+   </para>
+  </sect2>
+ </sect1>
+</chapter>
diff --git a/doc/src/sgml/fdwhandler.sgml b/doc/src/sgml/fdwhandler.sgml
index d1194def82..8cd3552bd3 100644
--- a/doc/src/sgml/fdwhandler.sgml
+++ b/doc/src/sgml/fdwhandler.sgml
@@ -1657,6 +1657,117 @@ ReparameterizeForeignPathByChild(PlannerInfo *root, List *fdw_private,
     </para>
    </sect2>
 
+   <sect2 id="fdw-callbacks-transaction-management">
+    <title>FDW Routines For Transaction Management</title>
+
+    <para>
+     Transaction management callbacks are used to commit, rollback, and
+     prepare the foreign transaction. If an FDW wishes that its foreign
+     transaction is managed by <productname>PostgreSQL</productname>'s global
+     transaction manager it must provide both
+     <function>CommitForeignTransaction</function> and
+     <function>RollbackForeignTransaction</function>. In addition, if an FDW
+     wishes to support <firstterm>atomic commit</firstterm> (as described in
+     <xref linkend="fdw-transaction-managements"/>), it must provide
+     <function>PrepareForeignTransaction</function> as well and can provide
+     <function>GetPrepareId</function> callback optionally.
+    </para>
+
+    <para>
+<programlisting>
+void
+PrepareForeignTransaction(FdwXactInfo *finfo);
+</programlisting>
+    Prepare the transaction on the foreign server. This function is called at the
+    pre-commit phase of the local transactions if foreign twophase commit is
+    required. This function is used only for distributed transaction management
+    (see <xref linkend="distributed-transaction"/>).
+    </para>
+
+    <para>
+     Note that this callback function is always executed by backend processes.
+    </para>
+    <para>
+<programlisting>
+bool
+CommitForeignTransaction(FdwXactInfo *finfo);
+</programlisting>
+    Commit the foreign transaction. This function is called either at
+    the pre-commit phase of the local transaction if the transaction
+    can be committed in one-phase or at the post-commit phase if
+    two-phase commit is required. If <literal>finfo-&gt;flags</literal> has
+    the flag <literal>FDW_XACT_FLAG_ONEPHASE</literal> the transaction
+    can be committed in one-phase, this function must commit the prepared
+    transaction identified by <literal>finfo-&gt;identifier</literal>.
+    </para>
+
+    <para>
+     Note that all cases except for calling <function>pg_resolve_fdwxact</function>
+     SQL function, this callback function is executed by foreign transaction
+     resolver processes.
+    </para>
+    <para>
+<programlisting>
+bool
+RollbackForeignTransaction(FdwXactInfo *finfo);
+</programlisting>
+    Rollback the foreign transaction. This function is called either at
+    the end of the local transaction after rolled back locally. The foreign
+    transactions are rolled back when a user requested rollbacking or when
+    an error occurs during the transaction. This function must be tolerant to
+    being called recursively if any error occurs during rollback of the foreign
+    transaction. So you would need to track recursion and prevent being called
+    infinitely. If <literal>finfo-&gt;flags</literal> has the flag
+    <literal>FDW_XACT_FLAG_ONEPHASE</literal> the transaction can be rolled
+    back in one-phase, otherwise this function must rollback the prepared
+    transaction identified by <literal>finfo-&gt;identifier</literal>.
+    </para>
+
+    <para>
+     The foreign transaction identified by <literal>finfo-&gt;identifier</literal>
+     might not exist on the foreign servers. This can happen when, for instance,
+     there is a failure during preparing the foreign transaction. Therefore, this
+     function needs to tolerate the undefined object error
+     (<literal>ERRCODE_UNDEFINED_OBJECT</literal>) rather than raising an error.
+    </para>
+
+    <para>
+     Note that in all cases except for calling <function>pg_resolve_fdwxact</function>
+     SQL function, this callback function is executed by foreign transaction
+     resolver processes.
+    </para>
+    <para>
+<programlisting>
+char *
+GetPrepareId(TransactionId xid, Oid serverid, Oid userid, int *prep_id_len);
+</programlisting>
+    Return null-terminated string that represents prepared transaction identifier
+    with its length <varname>*prep_id_len</varname>.
+    This optional function is called during executor startup for once per the
+    foreign server. Note that the transaction identifier must be a string literal,
+    less than <symbol>NAMEDATALEN</symbol> bytes long and should not be same
+    as any other concurrent prepared transaction id. If this callback routine
+    is not supported, <productname>PostgreSQL</productname>'s  distributed
+    transaction manager generates a unique identifier in the form of
+    <literal>fx_&lt;random value up to 2<superscript>31</superscript>&gt;_&lt;xid&gt;_&lt;user mapping oid&gt;</literal>.
+    </para>
+
+    <para>
+     Note that this callback function is always executed by backend processes.
+    </para>
+
+    <note>
+     <para>
+      Functions <function>PrepareForeignTransaction</function>,
+      <function>CommitForeignTransaction</function> and
+      <function>RolblackForeignTransaction</function> are called
+      outside of a valid transaction state. So please note that
+      you cannot use functions that use the system catalog cache
+      such as Foreign Data Wrapper helper functions described in
+      <xref linkend="fdw-helpers"/>.
+     </para>
+    </note>
+   </sect2>
    </sect1>
 
    <sect1 id="fdw-helpers">
@@ -2136,4 +2247,138 @@ GetForeignServerByName(const char *name, bool missing_ok);
 
   </sect1>
 
+  <sect1 id="fdw-transaction-managements">
+   <title>Transaction managements for Foreign Data Wrappers</title>
+   <para>
+    If a server used by an  FDW supports transactions, it is usually worthwhile
+    for the FDW to manage transactions opened on the foreign server. The FDW
+    callback function <literal>CommitForeignTransaction</literal>,
+    <literal>RollbackForeignTransaction</literal> and
+    <literal>PrepareForeignTransaction</literal> are used for transaction
+    management and must fit into the working of the
+    <productname>PostgreSQL</productname> transaction processing.
+   </para>
+
+   <para>
+    The information in <literal>FdwXactInfo</literal> can be used to get
+    information of foreign server being processed such as
+    <structname>ForeignServer</structname> and <structname>UserMapping</structname>
+    The <literal>flags</literal> has contains flag bit describing the
+    foreign transaction state for transaction management.
+   </para>
+
+   <para>
+    The foreign transaction needs to be registered to
+    <productname>PostgreSQL</productname> global transaction manager.
+    Registration and unregistration are done by calling
+    <function>FdwXactRegisterXact</function> and
+    <function>FdwXactUnregisterXact</function> respectively.
+    The FDW can pass a boolean <literal>modified</literal> along with
+    <structname>UserMapping</structname> to <function>FdwXactRegisterXact</function>
+    indicating that writes are going to happen on the foreign server.  Such
+    foreign servers are taken into account for the decision of two-phase
+    commit protocol being required or not.
+   </para>
+
+   <para>
+    The FDW callback function <function>CommitForeignTransaction</function>
+    and <function>RollbackForeignTransaction</function> are used to commit
+    and rollback foreign transactions. During transaction commit, the global
+    transaction manager calls <function>CommitForeignTransaction</function> function
+    in the pre-commit phase and calls
+    <function>RollbackForeignTransaction</function> function in the post-rollback
+    phase.
+   </para>
+
+   <para>
+    In addition to simply commit and rollback foreign transactions,
+    <productname>PostgreSQL</productname> global transaction manager enables
+    distributed transactions to atomically commit and rollback among all foreign
+    servers, which is as known as atomic commit in literature. To achieve atomic
+    commit, <productname>PostgreSQL</productname> employs two-phase commit
+    protocol, which is a type of atomic commitment protocol. Every FDWs that wish
+    to support two-phase commit protocol are required to have the FDW callback
+    function <function>PrepareForeignTransaction</function> and optionally
+    <function>GetPrepareId</function>, in addition to
+    <function>CommitForeignTransaction</function> and
+    <function>RollbackForeignTransaction</function>
+    (see <xref linkend="fdw-callbacks-transaction-management"/> for details).
+   </para>
+
+   <para>
+    An example of a distributed transaction is as follows
+<programlisting>
+BEGIN;
+UPDATE ft1 SET col = 'a';
+UPDATE ft2 SET col = 'b';
+COMMIT;
+</programlisting>
+    ft1 and ft2 are foreign tables on different foreign servers may be using different
+    Foreign Data Wrappers.
+   </para>
+
+   <para>
+    When the core executor access the foreign servers, foreign servers whose FDW
+    supports transaction management callback routines is registered as a participant.
+    During registration, <function>GetPrepareId</function> is called if provided to
+    generate a unique transaction identifier.
+   </para>
+
+   <para>
+    During pre-commit phase of the local transaction, the foreign transaction manager
+    persists the foreign transaction information to the disk and WAL, and then
+    prepare all foreign transactions by calling
+    <function>PrepareForeignTransaction</function> if two-phase commit protocol
+    is required. Two-phase commit is required when the transaction modified data
+    on more than one server including the local server itself and a user requests
+    foreign twophase commit (see <xref linkend="guc-foreign-twophase-commit"/>).
+   </para>
+
+   <para>
+    <productname>PostgreSQL</productname> commits locally and go to the next
+    step if and only if all foreign transactions are prepared successfully.
+    If any failure happens or a user requests to cancel during preparation,
+    the global transaction manager changes over rollback and calls
+    <function>RollbackForeignTransaction</function>.
+   </para>
+
+   <para>
+    When changing over rollback due to any failure, it calls
+    <function>RollbackForeignTransaction</function> with
+    <literal>FDWXACT_FLAG_ONEPHASE</literal> for foreign transactions which are not
+    closed yet and calls <function>RollbackForeignTransaction</function> without
+    that flag for foreign transactions which are already prepared.  For foreign
+    transactions which are being prepared, it does both because it's not sure that
+    the preparation has been completed on the foreign server. Therefore,
+    <function>RollbackForeignTransaction</function> needs to tolerate the undefined
+    object error.
+   </para>
+
+   <para>
+    Note that when <literal>(finfo-&gt;flags &amp; FDWXACT_FLAG_ONEPHASE)</literal>
+    is true, both <literal>CommitForeignTransaction</literal> function and
+    <literal>RollbackForeignTransaction</literal> function should commit and
+    rollback directly, rather than processing prepared transactions. This can
+    happen when two-phase commit is not required or a foreign server is not
+    modified within the transaction.
+   </para>
+
+   <para>
+    Once all foreign transactions are prepared, the core transaction manager commits
+    locally. After that the transaction commit waits for all prepared foreign
+    transaction to be committed before completion. After all prepared foreign
+    transactions are resolved the transaction commit completes.
+   </para>
+
+   <para>
+    One foreign transaction resolver process is responsible for foreign
+    transaction resolution on a database. The foreign transaction resolver process
+    calls either <function>CommitForeignTransaction</function> or
+    <function>RollbackForeignTransaction</function> to resolve the foreign
+    transaction identified by <literal>finfo-&gt;identifier</literal>. If failed
+    to resolve, the resolver process will exit with an error message. The foreign
+    transaction launcher will launch the resolver process again at
+    <xref linkend="guc-foreign-transaction-resolution-rety-interval"/> interval.
+   </para>
+  </sect1>
  </chapter>
diff --git a/doc/src/sgml/filelist.sgml b/doc/src/sgml/filelist.sgml
index 596bfecf8e..74aa15e705 100644
--- a/doc/src/sgml/filelist.sgml
+++ b/doc/src/sgml/filelist.sgml
@@ -49,6 +49,7 @@
 <!ENTITY wal           SYSTEM "wal.sgml">
 <!ENTITY logical-replication    SYSTEM "logical-replication.sgml">
 <!ENTITY jit    SYSTEM "jit.sgml">
+<!ENTITY distributed-transaction    SYSTEM "distributed-transaction.sgml">
 
 <!-- programmer's guide -->
 <!ENTITY bgworker   SYSTEM "bgworker.sgml">
diff --git a/doc/src/sgml/func.sgml b/doc/src/sgml/func.sgml
index 6388385edc..55bde2bdba 100644
--- a/doc/src/sgml/func.sgml
+++ b/doc/src/sgml/func.sgml
@@ -27269,6 +27269,153 @@ SELECT convert_from(pg_read_binary_file('file_in_utf8.txt'), 'UTF8');
 
   </sect2>
 
+  <sect2 id="functions-data-sanity">
+   <title>Data Sanity Functions</title>
+
+   <para>
+    The functions shown in <xref linkend="functions-data-sanity-table"/>
+    provide ways to check the sanity of data files in the cluster.
+   </para>
+
+   <table id="functions-data-sanity-table">
+    <title>Data Sanity Functions</title>
+    <tgroup cols="1">
+     <thead>
+      <row>
+       <entry role="func_table_entry"><para role="func_signature">
+        Function
+       </para>
+       <para>
+        Description
+       </para></entry>
+      </row>
+     </thead>
+
+     <tbody>
+      <row>
+       <entry role="func_table_entry"><para role="func_signature">
+        <indexterm>
+         <primary>pg_relation_check_pages</primary>
+        </indexterm>
+        <function>pg_relation_check_pages</function> ( <parameter>relation</parameter> <type>regclass</type> [, <parameter>fork</parameter> <type>text</type> ] )
+        <returnvalue>setof record</returnvalue>
+        ( <parameter>path</parameter> <type>text</type>,
+        <parameter>failed_block_num</parameter> <type>bigint</type> )
+       </para>
+       <para>
+        Checks the pages of the specified relation to see if they are valid
+        enough to safely be loaded into the server's shared buffers.  If
+        given, <parameter>fork</parameter> specifies that only the pages of
+        the given fork are to be verified.  <parameter>fork</parameter> can
+        be <literal>main</literal> for the main data
+        fork, <literal>fsm</literal> for the free space
+        map, <literal>vm</literal> for the visibility map,
+        or <literal>init</literal> for the initialization fork.  The
+        default of <literal>NULL</literal> means that all forks of the
+        relation should be checked.  The function returns a list of block
+        numbers that appear corrupted along with the path names of their
+        files.  Use of this function is restricted to superusers by
+        default, but access may be granted to others
+        using <command>GRANT</command>.
+       </para></entry>
+      </row>
+     </tbody>
+    </tgroup>
+   </table>
+
+  </sect2>
+
+  <sect2 id="functions-foreign-transaction">
+   <title>Foreign Transaction Management Functions</title>
+
+   <indexterm>
+    <primary>pg_resolve_foreign_xact</primary>
+   </indexterm>
+   <indexterm>
+    <primary>pg_remove_foreign_xact</primary>
+   </indexterm>
+
+   <para>
+    <xref linkend="functions-fdw-transaction-control-table"/> shows the functions
+    available for foreign transaction management.
+    These functions cannot be executed during recovery. Use of these function
+    is restricted to superusers.
+   </para>
+
+   <table id="functions-fdw-transaction-control-table">
+    <title>Foreign Transaction Management Functions</title>
+    <tgroup cols="3">
+     <thead>
+      <row><entry>Name</entry> <entry>Return Type</entry> <entry>Description</entry></row>
+     </thead>
+
+     <tbody>
+      <row>
+       <entry>
+        <literal><function>pg_resolve_foreign_xact(<parameter>transaction</parameter> <type>xid</type>, <parameter>userid</parameter> <type>oid</type>, <parameter>userid</parameter> <type>oid</type>)</function></literal>
+       </entry>
+       <entry><type>bool</type></entry>
+       <entry>
+        Resolve a foreign transaction. This function searches for foreign
+        transaction matching the arguments and resolves it. Once the foreign
+        transaction is resolved successfully, this function removes the
+        corresponding entry from <xref linkend="view-pg-foreign-xacts"/>.
+        This function won't resolve a foreign transaction which is being
+        processed.
+       </entry>
+      </row>
+      <row>
+       <entry>
+        <literal><function>pg_remove_foreign_xact(<parameter>transaction</parameter> <type>xid</type>, <parameter>serverid</parameter> <type>oid</type>, <parameter>userid</parameter> <type>oid</type>)</function></literal>
+       </entry>
+       <entry><type>void</type></entry>
+       <entry>
+        This function works the same as <function>pg_resolve_foreign_xact</function>
+        except that this removes the foreign transaction entry without resolution.
+        This function is useful to remove a foreign transaction entry whose foreign
+        server is no longer available.
+       </entry>
+      </row>
+     </tbody>
+    </tgroup>
+   </table>
+
+   <para>
+   The function shown in <xref linkend="functions-fdwxact-resolver-control-table"/>
+   control the foreign transaction resolvers.
+   </para>
+
+   <table id="functions-fdwxact-resolver-control-table">
+    <title>Foreign Transaction Resolver Control Functions</title>
+    <tgroup cols="3">
+     <thead>
+      <row>
+       <entry>Name</entry> <entry>Return Type</entry> <entry>Description</entry>
+      </row>
+     </thead>
+
+     <tbody>
+      <row>
+       <entry>
+        <literal><function>pg_stop_fdwxact_resolver(<parameter>dbid</parameter> <type>oid</type>)</function></literal>
+       </entry>
+       <entry><type>bool</type></entry>
+       <entry>
+        Stop the foreign transaction resolver running on the given database.
+        This function is useful for stopping a resolver process on the database
+        that you want to drop.
+       </entry>
+      </row>
+     </tbody>
+    </tgroup>
+   </table>
+
+   <para>
+    <function>pg_stop_fdwxact_resolver</function> is useful to be used before
+    dropping the database to that the foreign transaction resolver is connecting.
+   </para>
+
+  </sect2>
   </sect1>
 
   <sect1 id="functions-trigger">
diff --git a/doc/src/sgml/monitoring.sgml b/doc/src/sgml/monitoring.sgml
index dcbb10fb6f..94006d0b2a 100644
--- a/doc/src/sgml/monitoring.sgml
+++ b/doc/src/sgml/monitoring.sgml
@@ -1094,6 +1094,18 @@ postgres   27093  0.0  0.0  30096  2752 ?        Ss   11:34   0:00 postgres: ser
       <entry><literal>CheckpointerMain</literal></entry>
       <entry>Waiting in main loop of checkpointer process.</entry>
      </row>
+     <row>
+      <entry><literal>FdwXactLauncherMain</literal></entry>
+      <entry>Waiting in main loop of foreign transaction resolution launcher process.</entry>
+     </row>
+     <row>
+      <entry><literal>FdwXactResolverMain</literal></entry>
+      <entry>Waiting in main loop of foreign transaction resolution worker process.</entry>
+     </row>
+     <row>
+      <entry><literal>LogicalLauncherMain</literal></entry>
+      <entry>Waiting in main loop of logical launcher process.</entry>
+     </row>
      <row>
       <entry><literal>LogicalApplyMain</literal></entry>
       <entry>Waiting in main loop of logical replication apply process.</entry>
@@ -1318,6 +1330,18 @@ postgres   27093  0.0  0.0  30096  2752 ?        Ss   11:34   0:00 postgres: ser
       <entry><literal>DataFileWrite</literal></entry>
       <entry>Waiting for a write to a relation data file.</entry>
      </row>
+     <row>
+      <entry><literal>FdwXactFileRead</literal></entry>
+      <entry>Waiting for a read of a foreign transaction state file.</entry>
+     </row>
+     <row>
+      <entry><literal>FdwXactFileSync</literal></entry>
+      <entry>Waiting for a foreign transaction state file to reach stable storage.</entry>
+     </row>
+     <row>
+      <entry><literal>FdwXactFileWrite</literal></entry>
+      <entry>Waiting for a write of a foreign transaction state file.</entry>
+     </row>
      <row>
       <entry><literal>LockFileAddToDataDirRead</literal></entry>
       <entry>Waiting for a read while adding a line to the data directory lock
@@ -1624,6 +1648,11 @@ postgres   27093  0.0  0.0  30096  2752 ?        Ss   11:34   0:00 postgres: ser
       <entry>Waiting for activity from a child process while
        executing a <literal>Gather</literal> plan node.</entry>
      </row>
+     <row>
+      <entry><literal>FdwXactResolution</literal></entry>
+      <entry>Waiting for all foreign transaction participants to be resolved during
+       atomic commit among foreign servers.</entry>
+     </row>
      <row>
       <entry><literal>HashBatchAllocate</literal></entry>
       <entry>Waiting for an elected Parallel Hash participant to allocate a hash
@@ -1942,6 +1971,19 @@ postgres   27093  0.0  0.0  30096  2752 ?        Ss   11:34   0:00 postgres: ser
       <entry>Waiting to read or update dynamic shared memory allocation
        information.</entry>
      </row>
+     <row>
+      <entry><literal>FdwXactLock</literal></entry>
+      <entry>Waiting to read or update the state of foreign transactions.</entry>
+     </row>
+     <row>
+      <entry><literal>FdwXactResolutionLock</literal></entry>
+      <entry>Waiting to read or update information of foreign transaction
+       resolution.</entry>
+     </row>
+     <row>
+      <entry><literal>LogicalRepWorkerLock</literal></entry>
+      <entry>Waiting for action on logical replication worker to finish.</entry>
+     </row>
      <row>
       <entry><literal>LockFastPath</literal></entry>
       <entry>Waiting to read or update a process' fast-path lock
diff --git a/doc/src/sgml/postgres.sgml b/doc/src/sgml/postgres.sgml
index d453be3909..eca35c4a84 100644
--- a/doc/src/sgml/postgres.sgml
+++ b/doc/src/sgml/postgres.sgml
@@ -171,6 +171,7 @@ break is not needed in a wider output rendering.
   &wal;
   &logical-replication;
   &jit;
+  &distributed-transaction;
   &regress;
 
  </part>
diff --git a/doc/src/sgml/storage.sgml b/doc/src/sgml/storage.sgml
index 7136bbe7a3..d1fc367dfc 100644
--- a/doc/src/sgml/storage.sgml
+++ b/doc/src/sgml/storage.sgml
@@ -83,6 +83,12 @@ Item
   subsystem</entry>
 </row>
 
+<row>
+ <entry><filename>pg_fdwxact</filename></entry>
+ <entry>Subdirectory containing files used by the distributed transaction
+  manager subsystem</entry>
+</row>
+
 <row>
  <entry><filename>pg_logical</filename></entry>
  <entry>Subdirectory containing status data for logical decoding</entry>
diff --git a/src/backend/access/transam/README.fdwxact b/src/backend/access/transam/README.fdwxact
new file mode 100644
index 0000000000..ee3a2b3614
--- /dev/null
+++ b/src/backend/access/transam/README.fdwxact
@@ -0,0 +1,142 @@
+src/backend/access/transam/README.fdwxact
+
+Atomic Commit for Distributed Transactions
+===========================================
+
+The atomic commit feature enables us to commit and rollback either all of
+foreign servers or nothing. This ensures that the database data is always left
+in a conssitent state in term of federated database.
+
+
+Commit Sequence of Global Transactions
+--------------------------------
+
+We employee two-phase commit protocol to achieve commit among all foreign
+servers atomically. The sequence of distributed transaction commit consisnts
+of the following four steps:
+
+1. Foriegn Transaction Registration
+FDW implementation can register the transaction that opened on the foreign server
+(foreign transaction) to the group of the distributed transaction by calling
+FdwXactRegisterEntry() function.  The foreign transaction are managed until
+the end of the transaction by PostgreSQL's distributed transaction manager.
+
+2. Pre-Commit phase (1st phase of two-phase commit)
+The two-phase commit is required only if the transaction modified two or more
+servers including the local node.
+
+In this step, we write WAL record (XLOG_FDWXACT_INSERT) for all foreign
+transactions indicating that the foreign server is involved with the current
+transaction before doing PREPARE all foreign transactions.  Also, we wait for
+those WAL records to be replicated to the standby if synchronous replication is
+enabled.  After that we prepare all foreign transactions on the foreign server.
+Both writing the WAL records and waiting for synchronous replication must be
+done before actual preparing foreign transactions.  Because if we prepare all
+foerign transactions first then write the WAL records and synchronous replication,
+in cases where the server crashes between preparing the foreign transaction and
+writing the WAL record, we will end up losing foreign transaction after the
+crash recovery. Similarly, if the failover also happens before the replica
+receives the records, we will end up losing them on the new primary server.
+
+In cases where we could not prepare one foreign transaction for some reason,
+we error out and change to rollback.  We rollback foreign transaction that is
+not prepared yet and leave other already-prepared foreign transactions as a
+in-doubt transaction to the resolver process.
+
+3. Commit locally
+Once we've prepared all of them, commit the transaction locally.
+
+Once we've committed the local transaction (to be xact, right after flushing
+the commit WAL record to the disk), the outcome of the distributed transaction
+is determined and must not be changed.  All foreign prepared transactions must
+be committed anyway.
+
+4. Post-Commit Phase (2nd phase of two-phase commit)
+The steps so far are done by the backend process committing the transaction but
+this step (commit or rollback) is performed asynchronously by the foreign
+transaction resolver process.
+
+
+Identifying Foreign Transactions In GTM
+---------------------------------------
+
+To identify foreign transaction participants (as well as FdwXact entries) there
+are two ways: using {server OID, user OID} and using user mapping OID. The same
+is true for FDWs to identify the connections (and transactions upon) to the
+foreign server. We need to consider the case where the way to identify the
+transactions is not matched between GTM and FDWs, because the problem might occur
+when the user modifies the same foreign server by different roles within the
+transaction. For example, consider the following execution:
+
+BEGIN;
+SET ROLE user_A;
+INSERT INTO ft1 VALUES (1);
+SET ROLE user_B;
+INSERT INTO ft1 VALUES (1);
+COMMIT;
+
+For example, suppose that an FDW identifies the connection by {server OID, user OID}
+and GTM identifies the transactions by user mapping OID, and user_A and user_B use
+the public user mapping to connect server_X. In the FDW, there are two
+connections: {user_A, sever_X} and {user_B, server_X}, and therefore opens two
+transactions on each connection, while GTM has only one FdwXact entry because the two
+connections refer to the same user mapping OID. As a result, at the end of the
+transaction, GTM ends only one foreign transaction, leaving another one.
+
+On the other hand, suppose that an FDW identifies the connection by user mapping OID
+and GTM does that by {server OID, user OID}, the FDW uses only one connection and opens
+a transaction since both users refer to the same user mapping OID (we expect FDWs
+not to register the foreign transaction when not starting a new transaction on the
+foreign server). Since GTM also has one entry it can end the foreign transaciton
+properly. The downside would be that the user OID of FdwXact (i.g., FdwXact->userid)
+is the user who registered the foreign transaction for the first time, necessarily
+not the user who executed COMMIT.  For example in the above case, FdwXact->userid
+will be user_A, not user_B. But it’s not big problem in practice.
+
+Therefore, in fdwxact.c, we identify the foreign transaction by
+{server OID, user OID}.
+
+Foreign Transactions Status
+----------------------------
+
+Every foreign transaction has an FdwXact entry. When preparing a foreign
+transaction a FdwXact entry of which status starts from FDWXACT_STATUS_PREPARING
+are created with WAL logging. The status changes to FDWXACT_STATUS_PREPARED
+after the foreign transaction is prepared. And the status changes to
+FDWXACT_STATUS_COMMITTING and FDWXACT_STATUS_ABORTING before committing and
+aborting respectively. FdwXact entry is removed with WAL logging after resolved.
+
+FdwXact entries recovered during the recovery are marked as in-doubt if the
+corresponding local transaction is not prepared transaction. The initial
+status for those entries is FDWXACT_STATUS_PREPARED if they are recovered
+from WAL. Because we WAL logs only when preparing the foreign transaction we
+cannot know the exact fate of the foreign transaction from the recovery.
+
+The foreign transaction status transition is illustrated by the following
+graph describing the FdwXact->status:
+
+ +----------------------------------------------------+
+ |                      INVALID                       |
+ +----------------------------------------------------+
+    |                      |                       |
+    |                      v                       |
+    |           +---------------------+            |
+   (*1)         |      PREPARING      |           (*1)
+    |           +---------------------+            |
+    |                      |                       |
+    v                      v                       v
+ +----------------------------------------------------+
+ |                      PREPARED                      |
+ +----------------------------------------------------+
+           |                               |
+           v                               v
+ +--------------------+          +--------------------+
+ |     COMMITTING     |          |      ABORTING      |
+ +--------------------+          +--------------------+
+           |                               |
+           v                               v
+ +----------------------------------------------------+
+ |                        END                         |
+ +----------------------------------------------------+
+
+(*1) Paths for recovered FdwXact entries
-- 
2.24.3 (Apple Git-128)

